summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
-rw-r--r--Makefile6
-rw-r--r--include/info.h38
-rw-r--r--include/sse_accel.h37
-rw-r--r--src/main.c (renamed from main.c)72
-rw-r--r--src/sse_accel.S45
5 files changed, 192 insertions, 6 deletions
diff --git a/Makefile b/Makefile
index 2000f5e..9ec939f 100644
--- a/Makefile
+++ b/Makefile
@@ -1,6 +1,8 @@
-CFLAGS = -pedantic
+CFLAGS = -pedantic -Iinclude/
+CFILES = src/main.c
+ASMFILES = src/sse_accel.S
CC = gcc
-bin/fobfuscate: main.c
+bin/fobfuscate: $(CFILES) $(ASMFILES)
mkdir -p $(@D)
$(CC) $(CFLAGS) $^ -o $@
diff --git a/include/info.h b/include/info.h
new file mode 100644
index 0000000..966e8ff
--- /dev/null
+++ b/include/info.h
@@ -0,0 +1,38 @@
+/*
+ * Copyright (c) 2023 Ian Marco Moffett and the VegaOS team.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of VegaOS nor the names of its
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#ifndef INFO_H
+#define INFO_H
+
+struct cpu_info {
+ uint8_t has_sse2 : 1;
+ uint8_t has_sse3 : 1;
+};
+
+#endif /* INFO_H */
diff --git a/include/sse_accel.h b/include/sse_accel.h
new file mode 100644
index 0000000..bb6e7d9
--- /dev/null
+++ b/include/sse_accel.h
@@ -0,0 +1,37 @@
+/*
+ * Copyright (c) 2023 Ian Marco Moffett and the VegaOS team.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of VegaOS nor the names of its
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
+
+#ifndef ACCEL_H
+#define ACCEL_H
+
+#include <stdint.h>
+
+void accel_invert128(uint64_t addr);
+
+#endif /* ACCEL_H */
diff --git a/src/sse_accel.S b/src/sse_accel.S
new file mode 100644
index 0000000..928caaf
--- /dev/null
+++ b/src/sse_accel.S
@@ -0,0 +1,45 @@
+/*
+ * Copyright (c) 2023 Ian Marco Moffett and the VegaOS team.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of VegaOS nor the names of its
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
+
+ .section .text
+ .globl accel_invert128
+
+ /*
+ * accel_invert128(uint64_t addr)
+ */
+ accel_invert128:
+ movq %rdi, %rax
+ movdqu (%rax), %xmm0
+
+ /* Set %xmm1 to all 1s and emulate NOT for %xmm0 */
+ pcmpeqb %xmm1, %xmm1
+ pxor %xmm0, %xmm1
+
+ movdqu %xmm0, (%rax)
+ retq
diff --git a/main.c b/src/main.c
index 44b2dab..1b023fc 100644
--- a/main.c
+++ b/src/main.c
@@ -30,13 +30,23 @@
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
+#include <stdbool.h>
#include <assert.h>
#include <unistd.h>
+#include <info.h>
+#if defined(__x86_64__)
+#include <sse_accel.h>
+#endif /* defined(__x86_64__) */
#if __BYTE_ORDER__ != __ORDER_LITTLE_ENDIAN__
#error "Big endian machines not supported yet"
#endif
+#define cpuid(level, a, b, c, d) \
+ __asm__ __volatile__ ("cpuid\n\t" \
+ : "=a" (a), "=b" (b), "=c" (c), "=d" (d) \
+ : "0" (level))
+
#define flip_block(TMP_VAR, TYPE, BUF, POS) \
TMP_VAR = *(TYPE *)&BUF[POS]; \
TMP_VAR = ~TMP_VAR; \
@@ -80,8 +90,48 @@ writeback_file(const char *fname, const char *buf, size_t buf_size)
fclose(fp);
}
+#if defined(__x86_64__)
+static inline bool
+is_sse3_supported(void)
+{
+ uint32_t ecx, unused;
+
+ cpuid(0x0000001, unused, unused, ecx, unused);
+ return (ecx & (1 << 0)) != 0;
+}
+
+static inline bool
+is_sse2_supported(void)
+{
+ uint32_t edx, unused;
+ cpuid(0x0000001, unused, unused, unused, edx);
+ return (edx & (1 << 26)) != 0;
+}
+
+static void
+amd64_cpu_tests(struct cpu_info *info)
+{
+ bool sse3_supported;
+
+ sse3_supported = false;
+
+ if (is_sse3_supported()) {
+ printf("[?]: SSE3 supported, may use as optimization\n");
+ sse3_supported = true;
+ info->has_sse3 = 1;
+ }
+
+ if (!sse3_supported) {
+ if (is_sse2_supported()) {
+ printf("[?]: SSE2 supported, may use as optimization\n");
+ info->has_sse2 = 1;
+ }
+ }
+}
+#endif /* defined(__x86_64__) */
+
static char *
-encrypt(char *buf, size_t buf_size)
+encrypt(const struct cpu_info *info, char *buf, size_t buf_size)
{
size_t current_pos;
size_t step;
@@ -90,13 +140,22 @@ encrypt(char *buf, size_t buf_size)
current_pos = 0;
step = 8; /* Start at 8 bytes (64 bits) */
+#if defined(__x86_64__)
+ if (info->has_sse2 || info->has_sse3) {
+ step += 8; /* Start at 128 bits */
+ }
+#endif /* defined(__x86_64__) */
+
while (current_pos < buf_size) {
/* Ensure we aren't over 8 bytes and a power of two */
if (step != 1) {
- assert((step & 1) == 0 && step <= 8);
+ assert((step & 1) == 0 && step <= 16);
}
switch (step) {
+ case 16:
+ accel_invert128((uintptr_t)buf + current_pos);
+ break;
case 8:
flip_block(tmp, uint64_t, buf, current_pos);
break;
@@ -125,14 +184,19 @@ main(int argc, const char **argv)
{
size_t buf_size;
char *buf;
+ struct cpu_info info = { 0 };
if (argc < 2) {
fprintf(stderr, "Usage: %s <file>\n", argv[0]);
return 1;
}
+#if defined(__x86_64__)
+ amd64_cpu_tests(&info);
+#endif /* __x86_64__ */
+
buf = read_file(argv[1], &buf_size);
- encrypt(buf, buf_size);
+ encrypt(&info, buf, buf_size);
writeback_file(argv[1], buf, buf_size);
free(buf);
@@ -90,13 +140,22 @@ encrypt(char *buf, size_t buf_size)
while (current_pos < buf_size) {
- /* Ensure we aren't over 8 bytes and a power of two */
+ /* Ensure we aren't over 16 bytes and a power of two */
if (step != 1) {